# $Id: Goodbye.pm,v 1.2 2004/05/15 12:21:16 rich Exp $

package Email::List::Subscription::Goodbye;

use strict;
use Email::List::Subscription;

sub match
{
    my ($message) = shift(@_);
    my $listname;
    my @subscriptions = ();
    my $subscription;
    my $subject;

    $subject = $message->header('Subject');
    $subject =~ s/^\s+//;
    $subject =~ s/\s+$//;
    $subject =~ s/\s+/ /g;

    if ($subject =~ m/^.*Goodbye from (\S+)$/i)
    {
	$listname = $1;
    }
    elsif ($subject =~ m/^.*You have been unsubscribed from the \"([^\"]+)\" mailing list.*$/i)
    {
	$listname = $1;
    }
    elsif ($subject =~ m/^.*You have been unsubscribed from the ([^\"]+) mailing list.*$/i)
    {
	$listname = $1;
    }
    elsif ($subject =~ m/^.*Unsubscribed from \"([^\"]+)\".*$/i)
    {
	$listname = $1;
    }

    if (defined($listname))
    {
	# Guess the domain, if not was found. Use the domain
	# of the sender of the subscription message.
	if ($listname !~ m/\@[^\@]+$/)
	{
	    my $from = $message->header('From');
	    my ($from_addr) = Mail::Address->parse($from);

	    $from = $from_addr->address();
	    $from =~ m/^.*(\@[^\@]+)$/;
	    $listname .= lc($1);
	}

	# TODO: Preserve case of mailbox. Harder than it sounds,
	# since the listname may have been arbitrarily capitalised
	# in the welcome message.
	#
	# We lowercase it here for consistency with other
	# Email::List::Subscription::*.
	$listname = lc($listname);

	# TODO: Error reporting: no listname; no to.
	my $to = $message->header('To');
	my $date = $message->header('Date');

	# Parse the to address. Ignore multiple to addresses.
	my ($to_addr) = Mail::Address->parse($to);

	if (defined($to_addr) && defined($date))
	{
	    # TODO: Preserve case of mailbox.
	    $to = lc($to_addr->address());

	    $subscription
		= new Email::List::Subscription($listname,
						$to,
						$Email::List::Subscription::UNSUBSCRIPTION,
						$date);
	    push(@subscriptions, $subscription);
	}
    }

    return @subscriptions;
}

1;
